from django.contrib import admin
from django.contrib.auth.models import User
from django.contrib.auth.admin import UserAdmin
from .models import Token


def reset_token(modeladmin, request, qs):
    for user in qs.select_related('auf_auth_token'):
        user.auf_auth_token.reset_token()
reset_token.short_description = 'Reset token'


def send_token_by_email(modeladmin, request, qs):
    for user in qs.select_related('auf_auth_token'):
        user.auf_auth_token.send_token_by_email(request)
send_token_by_email.short_description = 'Send token email'


class TokenInline(admin.StackedInline):
    extra = 1
    model = Token


class TokenUserAdmin(UserAdmin):
    def get_token(self, obj):
        return obj.auf_auth_token.value
    get_token.short_description = 'Login token'

    def queryset(self, request):
        return super(TokenUserAdmin, self).queryset(request).select_related('auf_auth_token')

    inlines = UserAdmin.inlines + [TokenInline]
    list_display = list(UserAdmin.list_display) + ['get_token']
    actions = UserAdmin.actions + [
        reset_token,
        send_token_by_email,
        ]


admin.site.unregister(User)
admin.site.register(User, TokenUserAdmin)
