from django.utils.functional import SimpleLazyObject
from django.contrib.auth.models import AnonymousUser
from django.contrib.auth import login, authenticate
from django.conf import settings
from .models import Token, AUTH_TOKEN_PARAM, ALLOW_UNSECURED_TOKEN_AUTH
from .backends import TokenAuthBackend


def get_user(request):
    user = None
    if (not hasattr(request, '_cached_user') or
        request._cached_user.is_anonymous()):

        if not request.is_secure() and not (
            settings.DEBUG or ALLOW_UNSECURED_TOKEN_AUTH):
            raise Exception(
                'Please ensure this is a secure HTTP request')

        token_str = None
        if AUTH_TOKEN_PARAM in request.GET:
            token_str = request.GET.get(AUTH_TOKEN_PARAM, None)

        user = AnonymousUser()

        if token_str:
            user = authenticate(token=token_str) or user
            if not user.is_anonymous():
                login(request, user)

        request._cached_user = user
        
    return request._cached_user


class TokenAuthentification(object):
    def process_request(self, request):
        assert hasattr(request, 'session'), (
            "The Token authentication middleware requires session "
            "middleware to be installed. Edit your MIDDLEWARE_CLASSES"
            " setting to insert "
            "'django.contrib.sessions.middleware.SessionMiddleware'.")

        if not hasattr(request, 'user') or request.user.is_anonymous():
            request.user = SimpleLazyObject(lambda: get_user(request))

    
