from django.conf import settings
from django.db import models
from django.utils.crypto import get_random_string
from django.contrib.auth import models as auth
from django.template import RequestContext
from django.template.loader import render_to_string
from django.core.mail.message import EmailMultiAlternatives


AUTH_TOKEN_PARAM = getattr(settings, 'AUTH_TOKEN_PARAM', 'auth_token')
ALLOW_UNSECURED_TOKEN_AUTH = getattr(
    settings, 'AUTH_ALLOW_UNSECURED_TOKEN_AUTH', False)

EMAIL_SUBJECT = getattr(
    settings, 'AUTH_TOKEN_EMAIL_SUBJECT', (
        'Your new token.'))
EMAIL_FROM = getattr(
    settings, 'AUTH_TOKEN_EMAIL_FROM', None)
EMAIL_TEMPLATE = getattr(
    settings, 'AUTH_TOKEN_EMAIL_TEMPLATE', (
        'token_auth/new_token_email.html'))
EMAIL_ALTERNATIVE_TEMPLATE = getattr(
    settings, 'AUTH_TOKEN_EMAIL_TEMPLATE', (
        'token_auth/new_token_email.txt'))
LOGIN_URL_TEMPLATE = getattr(
    settings, 'AUTH_TOKEN_EMAIL_TEMPLATE', (
        '%(protocol)s://%(host)s/?%(token_param)s=%(token)s'))


class Token(models.Model):
    value = models.CharField(
        max_length=48,
        default=lambda: get_random_string(48),
        )
    user = models.OneToOneField(
        auth.User,
        related_name='auf_auth_token')

    sent_by_email = models.BooleanField()

    def send_token_by_email(self, request, recipients=[]):

        url = LOGIN_URL_TEMPLATE % {
            'protocol': ('https' if (
                request.is_secure() or not
                ALLOW_UNSECURED_TOKEN_AUTH) else 'http'),
            'host': request.get_host(),
            'token_param': AUTH_TOKEN_PARAM,
            'token': self.value,
            }

        tpl_data = {
            'token': self,
            'url': url,
            }

        sender = EMAIL_FROM
        if not sender:
            sender = 'noreply@%s' % request.get_host()

        html_response = render_to_string(
            EMAIL_TEMPLATE,
            tpl_data,
            context_instance=RequestContext(request, {}),
            )

        text_response = render_to_string(
            EMAIL_ALTERNATIVE_TEMPLATE,
            tpl_data,
            context_instance=RequestContext(request, {}),
            )

        if not recipients:
            recipients = [self.user.email]

        if not self.sent_by_email:
            msg = EmailMultiAlternatives(
                EMAIL_SUBJECT,
                text_response,
                sender,
                recipients,
                )
            msg.attach_alternative(html_response, "text/html")
            msg.send()
            

    def reset_token(self, save=True):
        self.value = get_random_string(48)
        self.email_sent = False
        if save:
            self.save()

    def __unicode__(self):
        return self.value

